/****************************************************************
 *                 harmonic signal synthesis
 *                     NVCom-02T-1508PL8T                       *
 *      Демонстрирует работу с микросхемой 1508PL8T по SPI:     *
 *      Запись одного профиля и                                 *
 *      демонстрация синтеза гармонического сигнала (синусоиды).*
 *      Результатом исполнения этого примера является
 *      демонстрация синусоиды 50МГц и амплитудой ~1В
 *      Выводы разъемов X10 и XP8                               *
 *      модулей 1508ПЛ8Т(Радиокомп) и NVCom-02TEM-3U            *
 *      должный быть соединены следующим образом:
 *      X10         XP8
 *      SCK         12 (LCLKn)
 *      SDI(MOSI)   4  (LDAT[3])
 *      SCS         2  SS_0(LDAT[1])
 *      S23(MISO)   3  (LDAT[2])
 *      GND         14 (GND)
 *                                                              *
 ***************************************************************/



// файл со ссылками на макроопределения регистров, в том числе нужных нам CLK_EN, DIR_MFBSP1, GPIO_DR1
#include "multicore/nvcom02t.h"
// файл со ссылками адресов регистров микросхемы 1508ПЛ8Т и команд
#include "dds.h"

// установка длительности паузы в тактах частоты CPU, 1 секунда
#define TIMEOUT 100000000

// частота работы SPI в Гц
unsigned int SPI_FRQ = 6000000;
// частота работы процессора в герцах
#define FREQ 240000000

unsigned int SYNT1_read(unsigned int addr){

	MFBSP3.TX.data = COMMAND_SETAFT|(addr & 0xFFFF);

	while (!(MFBSP3.TSR.bits.TBE));
	 while (!(MFBSP3.CSR.data & 1<<4));     //ожидание получения новых данных
		return (MFBSP3.RX.data & 0xFFFF); // приходят данные от предыдущей команды, это не ответ на команду чтения
}


void SYNT1_write(unsigned int addr, unsigned int val){ // Функция записи в адресное пространство DDS

	MFBSP3.TX.data = COMMAND_SETA|(addr & 0xFFFF);
	while (!(MFBSP3.TSR.bits.TBE));

	MFBSP3.TX.data = COMMAND_WR|(val & 0xFFFF);
	while (!(MFBSP3.TSR.bits.TBE));

}

void SYNT1_RESET(){
	SYNT1_write(0x0,0x78); //RESET
}

void SPI_Init() {
	        MFBSP3.CSR.bits.SPI_I2S_EN = 1; // включен режим I2S/SPI Появляется 201 в TX

	        MFBSP3.DIR.bits.TD_DIR = 1;    // MOSI- выход. MISO - вход по умолчанию
	        MFBSP3.DIR.bits.TCS_DIR = 1;   // SS[0] - выход
	        MFBSP3.DIR.bits.TCLK_DIR =1;   // TSCK - выход
	        MFBSP3.DIR.bits.RCLK_DIR =0;   // tiu RSCK - формируется приемником (то есть, нами же)
	        MFBSP3.DIR.bits.RD_DIR = 0;    // tiu MISO - вход по умолчанию
// настройка передатчика
	        MFBSP3.TCTR.bits.SS_0 = 1;      // Используется SS[0] в качестве CS
	        MFBSP3.TCTR.bits.TWORDLEN = 23; // длина слова - 24 разряда
	        MFBSP3.TCTR.bits.TMBF = 1;      // старшим битом вперед
	        MFBSP3.TCTR.bits.TWORDCNT = 0; // одно слово во фрейме – чтобы сигнал SS[0]
						// переключался в единицу после передачи каждого слова
	        MFBSP3.TCTR.bits.TNEG = 1;      // TNEG=1, TDEL=1. При данном сочетании выдача данных
	        MFBSP3.TCTR.bits.TDEL = 1;      // происходит по заднему фронту TSCK
		                                                  // исходное состояние TSCK перед началом передачи - высокое.
	        MFBSP3.TCTR.bits.TMODE = 1;     // режим SPI (TMODE=0 – режим I2S)
	        MFBSP3.TCTR.bits.TPACK = 0;

	        MFBSP3.TCTR_RATE.data = (FREQ/(2*SPI_FRQ)) - 1; // частота работы SPI

// настройка приемника
	        MFBSP3.RCTR.bits.RWORDLEN = 23;// длина принимаемого слова - 24 разряда
	        MFBSP3.RCTR.bits.RMBF = 1;	   // старшим битом вперед
            MFBSP3.RCTR.bits.RWORDCNT=0; // 1 слово во фрейме

// RSCK и SS[0] - берутся от передатчика
	        MFBSP3.RCTR.bits.RCS_CP = 1;
	        MFBSP3.RCTR.bits.RCLK_CP = 1;

	        MFBSP3.RCTR.bits.RMODE = 1;    // режим SPI



	        MFBSP3.RCTR.bits.RNEG = 1;       // RNEG=1, RDEL=1. При данном сочетании захват данных при
	        MFBSP3.RCTR.bits.RDEL = 1;       // приеме данных происходит по переднему фронту RSCK
	        		        		                                         // а исходное состояние RSCK перед началом приема - высокое.
	        		        		                                         // RSCK и SS[0] - берутся от передатчика

	        MFBSP3.RCTR.bits.REN = 1;	   // приемник включен
	        MFBSP3.RSTART.data = 1;
	        MFBSP3.TSTART.data = 1;
	        MFBSP3.TCTR.bits.TEN = 1;       // TEN - передатчик включен


	                                       // Такой порядок включения рекомендован разработчиком
}

unsigned int GetCP0_Count() {
                         unsigned int result;
                         asm volatile ("mfc0 %0, $9" :"=r"(result));
                         return result;
}

void SetCP0_Count(unsigned int value) {
                 asm volatile ("mtc0 %0, $9" ::"r"(value));
}

unsigned int val_read = 0;
unsigned int val_read2 = 0;
unsigned int val_read3 = 0;

int main() {

	unsigned int rcv_array[24];
	unsigned int i;
	SPI_Init();

	SYNT1_write(SEL_REG,0x0000);
	SYNT1_write(CTR,0x1000);
	SYNT1_write(SYNC,0x0000);
	//SYNT1_write(T_CAPTURE,0x1BAD);
	SYNT1_write(TC_L,0x0000);
	SYNT1_write(TC_H,0x0000);
	SYNT1_write(CH1_TSW,0x0000);


	// Задается частота
	//Это частота 25МГц
	SYNT1_write(CH1_dPh0_L,0x0000);
	SYNT1_write(CH1_dPh0_M,0x0000);
	SYNT1_write(CH1_dPh0_H,0x0800);

	SYNT1_write(CH1_P0,0x0000);
	SYNT1_write(CH1_Mul0,0x7fff);
	SYNT1_write(CH1_Offset0,0x0000);

	// Ниже идет чтение записанных регистров
	rcv_array[0]=SYNT1_read(CH1_dPh0_H);
	rcv_array[1]=SYNT1_read(CH1_dPh0_H);
	rcv_array[2]=SYNT1_read(CH1_dPh0_H);
	rcv_array[3]=SYNT1_read(CH1_dPh0_H);
	rcv_array[4]=SYNT1_read(CH1_dPh0_H);
	rcv_array[5]=SYNT1_read(CH1_dPh0_H);
	rcv_array[6]=SYNT1_read(CH1_dPh0_H);
	rcv_array[7]=SYNT1_read(CH1_dPh0_H);
	rcv_array[8]=SYNT1_read(CH1_dPh0_H);
		rcv_array[9]=SYNT1_read(CH1_dPh0_H);
		rcv_array[10]=SYNT1_read(CH1_dPh0_H);
		rcv_array[11]=SYNT1_read(CH1_dPh0_H);
		rcv_array[12]=SYNT1_read(CH1_dPh0_H);
		rcv_array[13]=SYNT1_read(CH1_dPh0_H);
		rcv_array[14]=SYNT1_read(CH1_dPh0_H);
		rcv_array[15]=SYNT1_read(CH1_dPh0_H);
		rcv_array[16]=SYNT1_read(CH1_dPh0_H);
			rcv_array[17]=SYNT1_read(CH1_dPh0_H);
			rcv_array[18]=SYNT1_read(CH1_dPh0_H);
			rcv_array[19]=SYNT1_read(CH1_dPh0_H);
			rcv_array[20]=SYNT1_read(CH1_dPh0_H);
			rcv_array[21]=SYNT1_read(CH1_dPh0_H);
			rcv_array[22]=SYNT1_read(CH1_dPh0_H);
			rcv_array[23]=SYNT1_read(CH1_dPh0_H);




	while (1) ;

}
